#!/bin/bash
# Automatic generation of PBS job scripts
# Author: Le Yan, 09/19/2016

# Show usage information
usage()
{
  name=`basename $0`
  cat << Usageinfo

This script generates PBS job scripts for temperature values supplied via command line.

Usage: $name [arguments]

Arguments:
  -t:  temperature lower bound;
  -nt: number of temperature values;
  -dt: temperature interval;
  -a:  name of allocation;
  -v:  verbose mode;
  -h:  print this help page;    

Examples:
  $name -t 273;
	Run simulation with temp=273
  $name -t 273 -nt 3 -dt 10
	Run simulations with temp={273,283,293}

Usageinfo
}


# Generate job scripts
jobgen()
{
  cat > $scriptname << Jobscript

#!/bin/sh
#PBS -A $alloc
#PBS -q checkpt
#PBS -l nodes=1:ppn=16
#PBS -l walltime=72:00:00
#PBS -V
#PBS -j oe
#PBS -N $jobname

cd $jobdir
$program -temp $temp

Jobscript

  if [ $verbose == "1" ] ; then
    echo 
    echo "Creating the job script in $jobdir"
    echo 
  fi
}

# Set default values
program=myprogram
alloc=myallocation
temp=""
nt=""
dt=""
basedir=`pwd`
verbose=0

# Process command line arguments
while [ "$#" -gt 0 ] ; do
case "$1" in
  -t)
	temp="$2"
	shift 2;;
  -nt)
	nt="$2"
	shift 2;;
  -dt)
	dt="$2"
	shift 2;;  
  -a)
	alloc="$2"
	shift 2;;
  -v)
	verbose=1;
	shift;;
  -h)
	usage
	exit
	;;
  *)
	echo "Invalid option: $1"
	usage
	exit
	;;
esac
done

# Sanity checks

# Temperature must be specified.
if [ -z "$temp" ] ; then
  echo
  echo "Error: -t option is mandatory!"
  usage
  exit 1
fi

# Both dt and nt must be specified at the same time.
if [ -n "$nt" -a -z "$dt" ] || [ -n "$dt" -a -z "$nt" ] ; then
  echo
  echo "Error: parameters nt and dt must be defined together!"
  exit 1
fi

# Temp, dt and nt must be integers
re='^[0-9]+$'
if ! [[ $temp =~ $re ]] ; then
  echo
  echo "Error: Tempature is not a number! Check your argument list!"
  exit 1
fi
if [ -n "$dt" ] ; then
  if ! [[ $dt =~ $re ]] ; then
    echo
    echo "Error: dt is not a number! Check your argument list!"
    exit 1
  fi
fi
if [ -n "$nt" ] ; then
  if ! [[ $nt =~ $re ]] ; then
    echo
    echo "Error: nt is not a number! Check your argument list!"
    exit 1
  fi
fi

# Create the list of temprature values
templist=$temp
nsim=1
if [[ $nt -gt 1 ]] ; then
  for n in `seq 2 $nt` ; do
    newtemp=$[$temp+($n-1)*$dt]
    templist=$templist" $newtemp"
  done
  nsim=$nt
fi

# Print some useful information to screen
echo `date`
echo 
echo "A Total of $nsim simulations will be run under $basedir:"
echo "Temps = $templist"
echo 
echo "Using allocation $alloc"
echo 

# Create job scripts
for temp in $templist ; do
  jobdir="$basedir/Sim_T$temp"
  scriptname="$jobdir/job.T$temp.pbs"
  jobname="$program.T$temp"
  if [ $verbose == "1" ]; then
    echo "Script: "$scriptname
    echo "Job: "$jobname
  fi
  if ! [ -d $jobdir ] ; then
    mkdir $jobdir
  fi
  jobgen
done

echo "Done!"
echo `date`
echo 

